#include "PulseView.h"
#include <string.h>
#include <Alert.h>
#include <Window.h>
#include <stdio.h>
#include <Application.h>

void TPulseView::do_init ()
{	//init sensor object and quit if no good
	sensor =  new Mlm78();
	if (sensor->InitCheck() != B_OK) 
	{	(new BAlert("Error", "LM78 driver not present or already in use!", "Quit", NULL,  NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
		be_app->PostMessage(B_QUIT_REQUESTED);
		return;
	}
	sensor->Update();
	
	system_info si;
	get_system_info ( &si );
	numCPUs = si.cpu_count;
	
	fCPUs= new TProgressBar*[ numCPUs ];
	if(numCPUs==1)
	{	BRect theRect (TEMP_WIDTH, 0,TEMP_WIDTH+PROGRESS_WIDTH,PROGRESS_HEIGHT + PROGRESS_HEIGHT - 1);
		fCPUs[0] = new TProgressBar( theRect, "CPU Progress Bar");
		AddChild( fCPUs[0] );
	}
	else
		for(int i=0; i<numCPUs; i++ ) 
		{	BRect theRect (TEMP_WIDTH, PROGRESS_HEIGHT * i,TEMP_WIDTH+PROGRESS_WIDTH,PROGRESS_HEIGHT * i + PROGRESS_HEIGHT - 1);
			fCPUs[i] = new TProgressBar( theRect, "CPU Progress Bar");
			AddChild( fCPUs[i] );
		}
			
	lastUpdateTime = system_time();
	count=0;
	SetViewColor (0, 0, 0);
}


void TPulseView::DrawTemps()
{	char text[10];
	SetLowColor(0,0,0);				
	SetHighColor(0,0,0);
	FillRect(BRect(2, 2, TEMP_WIDTH-1, PROGRESS_HEIGHT * 2 + PROGRESS_HEIGHT - 1));	
	for(int i=0;i<3;i++)
	{	int level=sensor->TempCPU[i]-20;
		if(level<0) level=0;
		if(level>40) level=40;
		{	int r,g,b; //nice flow of colors
			if(level<20) r=15+level*3;else r=(level-20)*9+75;
			if(level<20) b=255-level*9;else b=75-(level-20)*3;
			if(level<20) g=140+level*5; else g=240-(level-20)*12;
			SetHighColor(r,g,b);	
		}
		sprintf(text, "%d°", sensor->TempCPU[i]);	
		DrawString(text, BPoint(2, PROGRESS_HEIGHT * i + PROGRESS_HEIGHT - 3));
	}
}

void TPulseView::DrawFans()
{	char text[10];	
	SetLowColor(0,0,0);				
	SetHighColor(0,0,0);
	FillRect(BRect(2+TEMP_WIDTH+PROGRESS_WIDTH, 2, TEMP_WIDTH+PROGRESS_WIDTH+FAN_WIDTH-2, PROGRESS_HEIGHT * 2 + PROGRESS_HEIGHT - 1));
	SetHighColor(200,200,200);
	for(int i=0;i<2;i++) if(sensor->FanCPU[i]>=0)
	{	sprintf(text, "%drpm", sensor->FanCPU[i]);
		DrawString(text, BPoint(TEMP_WIDTH+PROGRESS_WIDTH+2, PROGRESS_HEIGHT * i + PROGRESS_HEIGHT - 3));
	}
}

void TPulseView::DrawVolts()
{	char text[10];
	SetLowColor(0,0,0);				
	SetHighColor(0,0,0);
	FillRect(BRect(2+TEMP_WIDTH, PROGRESS_HEIGHT * 2 + 2, TEMP_WIDTH+PROGRESS_WIDTH-1, PROGRESS_HEIGHT * 2 + PROGRESS_HEIGHT - 1));
	SetHighColor(200,200,200);
	sprintf(text, "%.1fv", sensor->VoltP12);	
	DrawString(text, BPoint(TEMP_WIDTH+2, PROGRESS_HEIGHT * 2 + PROGRESS_HEIGHT - 3));
	sprintf(text, "%.2fv", sensor->VoltP5);	
	DrawString(text, BPoint(TEMP_WIDTH+2+35, PROGRESS_HEIGHT * 2 + PROGRESS_HEIGHT - 3));
	sprintf(text, "%.2fv", sensor->VoltM5);	
	DrawString(text, BPoint(TEMP_WIDTH+2+70, PROGRESS_HEIGHT * 2 + PROGRESS_HEIGHT - 3));
	sprintf(text, "%.1fv", sensor->VoltM12);	
	DrawString(text, BPoint(TEMP_WIDTH+2+105,PROGRESS_HEIGHT * 2 + PROGRESS_HEIGHT - 3));
}
extern const char* AppSig;
status_t TPulseView::Archive(BMessage *data, bool deep) const
{
	int i = BView::Archive (data, deep);
	data->AddString ("add_on", AppSig);
	return i;
}

BArchivable* TPulseView::Instantiate (BMessage *data)
{
	if (!validate_instantiation (data, "TPulseView"))
		return NULL;
	return new TPulseView (data);
}

void  TPulseView::AttachedToWindow()
{	Window()->SetPulseRate(200000);
}

void TPulseView::Pulse()
{	system_info fSysInfo;
	bigtime_t now = system_time();	//update activity
	double scaleFactor = (100 / 4.9) / (now - lastUpdateTime);
	lastUpdateTime = now;
	get_system_info (&fSysInfo);
	for(int i=0; i < numCPUs; i++ ) 
		fCPUs[i]->Update(fSysInfo.cpu_infos[i].active_time,scaleFactor);
	
	count=(count+1)%5;
	if(count==0)	
	{	sensor->Update();
		DrawTemps();
		DrawFans();
		DrawVolts();
	}
}

void TPulseView::Draw (BRect up)
{	BView::Draw(up);
	DrawTemps();
	DrawFans();
	DrawVolts();	
}

extern const char* AboutMiniPulse;
void TPulseView::MessageReceived (BMessage *message)
{
	if (message->what == B_ABOUT_REQUESTED)
		(new BAlert ("About MiniPulse", AboutMiniPulse, "OK"))->Go(NULL);
	else
		BView::MessageReceived (message);
}
